/* usb without crystal for MH55B digital compas designed by takefuji on Dec.13, 2009 */
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <math.h>
#include <compat/deprecated.h>
#include "usbdrv.h"
#include "oddebug.h"
#include <math.h>

static void timerInit(void)
{
    TCCR1 = 0x0b;           /* select clock: 16.5M/1k -> overflow rate = 16.5M/256k = 62.94 Hz */
}

PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = { /* USB report descriptor */
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x06,                    // USAGE (Keyboard)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
    0xc0                           // END_COLLECTION
};

void disable(){
sbi(DDRB,1);		 //PB1(/EN) output
sbi(PORTB,1);		//PB1=1 diabled
}

void enable(){
sbi(DDRB,1);		 //PB1(/EN) output
cbi(PORTB,1);		//PB1=0 enabled
}

void dout(char *pulse){
disable();
enable();
_delay_ms(1);
sbi(DDRB,4);		//PB4(DinDout) output
sbi(DDRB,3);		//PB3(CLK) output
sbi(PORTB,3);		//PB3(CLK)=1
_delay_ms(1);
while(*pulse){
if((*pulse)=='0'){
	_delay_ms(1);
	cbi(PORTB,4);	//PB4(DinDout)=0
	_delay_ms(1);
	cbi(PORTB,3);	//PB3(CLK)=0
	_delay_ms(1);
	sbi(PORTB,3);	//PB3(CLK)=1
	_delay_ms(1);
	pulse++;}
else{ _delay_ms(1);
	sbi(PORTB,4);	//PB4(DinDout)=1
	_delay_ms(1);
	cbi(PORTB,3);	//PB3(CLK)=0
	_delay_ms(1);
	sbi(PORTB,3);	//PB3(CLK)=1
	_delay_ms(1);
	pulse++;}
	}
}

unsigned long din(unsigned char bits){
unsigned char i;
unsigned long data=0;
enable();
cbi(DDRB,4);		//PB4(DinDout) input
sbi(PORTB,3);		//PB3(CLK)=1
_delay_ms(1);
for(i=0;i<bits;i++){
if((PINB&0x10)==0){data |=(0<<(bits-1-i));}
		else{data |=(1<<(bits-1-i));}
_delay_ms(1);
cbi(PORTB,3);		//PB3(CLK)=0
_delay_ms(1);
sbi(PORTB,3);		//PB3(CLK)=1
_delay_ms(1);
}
return data;
}


void read(){
unsigned long ret;
disable();
dout("0000");
_delay_ms(1);
dout("1000");
_delay_ms(1);
ret=0;
while(ret!=0x0c){
disable();
enable();
dout("1100");
ret=din(4);}
}

uchar usbFunctionSetup(uchar data[8])
{
static uchar replybuf[8];	
usbMsgPtr = replybuf;
unsigned char c=data[1];
unsigned int result;
int x,y;

if(c=='x'){
	read();
	result=din(11);
	x=(0x7ff & result);
if((x&0x400)!=0){x = x|0xf800;}
	result=din(11);
	y=(0x7ff & result);
if((y&0x400)!=0){y = y|0xf800;}
double	theta=atan2(-y,x);
	sprintf(replybuf,"%1.5f",theta);
//	sprintf(replybuf,"%s%d%s%d%s%1.5lf"," x=",x," y=",y," angle=",theta);
	return sizeof(theta);}
else {
replybuf[0]=c;return 1;}
}

/* ------------------------------------------------------------------------- */
/* ------------------------ Oscillator Calibration ------------------------- */
/* ------------------------------------------------------------------------- */

/* Calibrate the RC oscillator to 8.25 MHz. The core clock of 16.5 MHz is
 * derived from the 66 MHz peripheral clock by dividing. Our timing reference
 * is the Start Of Frame signal (a single SE0 bit) available immediately after
 * a USB RESET. We first do a binary search for the OSCCAL value and then
 * optimize this value with a neighboorhod search.
 * This algorithm may also be used to calibrate the RC oscillator directly to
 * 12 MHz (no PLL involved, can therefore be used on almost ALL AVRs), but this
 * is wide outside the spec for the OSCCAL value and the required precision for
 * the 12 MHz clock! Use the RC oscillator calibrated to 12 MHz for
 * experimental purposes only!
 */
static void calibrateOscillator(void)
{
uchar       step = 128;
uchar       trialValue = 0, optimumValue;
int         x, optimumDev, targetValue = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);

    /* do a binary search: */
    do{
        OSCCAL = trialValue + step;
        x = usbMeasureFrameLength();    /* proportional to current real frequency */
        if(x < targetValue)             /* frequency still too low */
            trialValue += step;
        step >>= 1;
    }while(step > 0);
    /* We have a precision of +/- 1 for optimum OSCCAL here */
    /* now do a neighborhood search for optimum value */
    optimumValue = trialValue;
    optimumDev = x; /* this is certainly far away from optimum */
    for(OSCCAL = trialValue - 1; OSCCAL <= trialValue + 1; OSCCAL++){
        x = usbMeasureFrameLength() - targetValue;
        if(x < 0)
            x = -x;
        if(x < optimumDev){
            optimumDev = x;
            optimumValue = OSCCAL;
        }
    }
    OSCCAL = optimumValue;
}
/*
Note: This calibration algorithm may try OSCCAL values of up to 192 even if
the optimum value is far below 192. It may therefore exceed the allowed clock
frequency of the CPU in low voltage designs!
You may replace this search algorithm with any other algorithm you like if
you have additional constraints such as a maximum CPU clock.
For version 5.x RC oscillators (those with a split range of 2x128 steps, e.g.
ATTiny25, ATTiny45, ATTiny85), it may be useful to search for the optimum in
both regions.
*/

void    usbEventResetReady(void)
{
    calibrateOscillator();
    eeprom_write_byte(0, OSCCAL);   /* store the calibrated value in EEPROM */
}


int main(void)
{ 
uchar i;
uchar   calibrationValue;

    calibrationValue = eeprom_read_byte(0); /* calibration value from last time */
    if(calibrationValue != 0xff){
        OSCCAL = calibrationValue;
    }
    odDebugInit();
    usbDeviceDisconnect();
    for(i=0;i<20;i++){  /* 300 ms disconnect */
        _delay_ms(15);
    }
    usbDeviceConnect();
    timerInit();
    usbInit();
    sei();
    for(;;){    /* main event loop */
        usbPoll();
        }
    return 0;
}

